package com.sun.jmx.mbeanserver;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.GenericArrayType;
import java.lang.annotation.Annotation;
import javax.management.openmbean.OpenMBeanOperationInfoSupport;
import javax.management.openmbean.OpenMBeanParameterInfo;
import javax.management.openmbean.OpenMBeanParameterInfoSupport;
import javax.management.MBeanParameterInfo;
import javax.management.MBeanOperationInfo;
import javax.management.openmbean.OpenType;
import javax.management.openmbean.OpenMBeanAttributeInfoSupport;
import javax.management.ImmutableDescriptor;
import javax.management.Descriptor;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Type;
import java.lang.reflect.Method;
import javax.management.NotCompliantMBeanException;

class MXBeanIntrospector extends MBeanIntrospector<ConvertingMethod>
{
    private static final MXBeanIntrospector instance;
    private final MBeanIntrospector.PerInterfaceMap<ConvertingMethod> perInterfaceMap;
    private static final MBeanIntrospector.MBeanInfoMap mbeanInfoMap;
    
    MXBeanIntrospector() {
        this.perInterfaceMap = new MBeanIntrospector.PerInterfaceMap();
    }
    
    static MXBeanIntrospector getInstance() {
        return MXBeanIntrospector.instance;
    }
    
    @Override
    MBeanIntrospector.PerInterfaceMap<ConvertingMethod> getPerInterfaceMap() {
        return this.perInterfaceMap;
    }
    
    @Override
    MBeanIntrospector.MBeanInfoMap getMBeanInfoMap() {
        return MXBeanIntrospector.mbeanInfoMap;
    }
    
    @Override
    MBeanAnalyzer<ConvertingMethod> getAnalyzer(Class<?> clazz) throws NotCompliantMBeanException {
        return MBeanAnalyzer.analyzer(clazz, this);
    }
    
    @Override
    boolean isMXBean() {
        return true;
    }
    
    @Override
    ConvertingMethod mFrom(Method method) {
        return ConvertingMethod.from(method);
    }
    
    @Override
    String getName(ConvertingMethod convertingMethod) {
        return convertingMethod.getName();
    }
    
    @Override
    Type getGenericReturnType(ConvertingMethod convertingMethod) {
        return convertingMethod.getGenericReturnType();
    }
    
    @Override
    Type[] getGenericParameterTypes(ConvertingMethod convertingMethod) {
        return convertingMethod.getGenericParameterTypes();
    }
    
    @Override
    String[] getSignature(ConvertingMethod convertingMethod) {
        return convertingMethod.getOpenSignature();
    }
    
    @Override
    void checkMethod(ConvertingMethod convertingMethod) {
        convertingMethod.checkCallFromOpen();
    }
    
    @Override
    Object invokeM2(ConvertingMethod convertingMethod, Object o, Object[] array, Object o2) throws InvocationTargetException, IllegalAccessException, MBeanException {
        return convertingMethod.invokeWithOpenReturn((MXBeanLookup)o2, o, array);
    }
    
    @Override
    boolean validParameter(ConvertingMethod convertingMethod, Object o, int n, Object o2) {
        if (o == null) {
            Type type = convertingMethod.getGenericParameterTypes()[n];
            return !(type instanceof Class) || !((Class)type).isPrimitive();
        }
        Object fromOpenParameter;
        try {
            fromOpenParameter = convertingMethod.fromOpenParameter((MXBeanLookup)o2, o, n);
        }
        catch (Exception ex) {
            return true;
        }
        return MBeanIntrospector.isValidParameter(convertingMethod.getMethod(), fromOpenParameter, n);
    }
    
    @Override
    MBeanAttributeInfo getMBeanAttributeInfo(String s, ConvertingMethod convertingMethod, ConvertingMethod convertingMethod2) {
        int n = convertingMethod != null;
        int n2 = convertingMethod2 != null;
        int n3 = n && this.getName(convertingMethod).startsWith("is");
        OpenType<Object> openReturnType;
        Type genericReturnType;
        if (!(n == 0)) {
            openReturnType = convertingMethod.getOpenReturnType();
            genericReturnType = convertingMethod.getGenericReturnType();
        }
        else {
            openReturnType = convertingMethod2.getOpenParameterTypes()[0];
            genericReturnType = convertingMethod2.getGenericParameterTypes()[0];
        }
        ImmutableDescriptor immutableDescriptor = typeDescriptor(openReturnType, genericReturnType);
        if (!(n == 0)) {
            immutableDescriptor = ImmutableDescriptor.union(new Descriptor[] { immutableDescriptor, convertingMethod.getDescriptor() });
        }
        if (!(n2 == 0)) {
            immutableDescriptor = ImmutableDescriptor.union(new Descriptor[] { immutableDescriptor, convertingMethod2.getDescriptor() });
        }
        if (canUseOpenInfo(genericReturnType)) {
            return new OpenMBeanAttributeInfoSupport(s, s, openReturnType, (boolean)n, (boolean)n2, (boolean)n3, immutableDescriptor);
        }
        return new MBeanAttributeInfo(s, originalTypeString(genericReturnType), s, (boolean)n, (boolean)n2, (boolean)n3, immutableDescriptor);
    }
    
    @Override
    MBeanOperationInfo getMBeanOperationInfo(String s, ConvertingMethod convertingMethod) {
        Method method = convertingMethod.getMethod();
        OpenType<Object> openReturnType = convertingMethod.getOpenReturnType();
        Type genericReturnType = convertingMethod.getGenericReturnType();
        OpenType[] openParameterTypes = convertingMethod.getOpenParameterTypes();
        Type[] genericParameterTypes = convertingMethod.getGenericParameterTypes();
        MBeanParameterInfo[] signature = new MBeanParameterInfo[openParameterTypes.length];
        int canUseOpenInfo = canUseOpenInfo(genericReturnType);
        boolean n = true;
        Annotation[][] parameterAnnotations = method.getParameterAnnotations();
        Object o;
        for (int i = 0; i < openParameterTypes.length; signature[i] = o, ++i) {
            String string = "p" + i;
            OpenType<Object> openType = openParameterTypes[i];
            Type type = genericParameterTypes[i];
            ImmutableDescriptor union = ImmutableDescriptor.union(new Descriptor[] { typeDescriptor(openType, type), Introspector.descriptorForAnnotations(parameterAnnotations[i]) });
            if (canUseOpenInfo(type)) {
                o = new OpenMBeanParameterInfoSupport(string, string, openType, union);
            }
            else {
                n = false;
                o = new MBeanParameterInfo(string, originalTypeString(type), string, union);
            }
        }
        ImmutableDescriptor union2 = ImmutableDescriptor.union(new Descriptor[] { typeDescriptor(openReturnType, genericReturnType), Introspector.descriptorForElement(method) });
        if (canUseOpenInfo && n) {
            OpenMBeanParameterInfo[] signature2 = new OpenMBeanParameterInfo[signature.length];
            System.arraycopy(signature, 0, signature2, 0, signature.length);
            return new OpenMBeanOperationInfoSupport(s, s, signature2, openReturnType, 3, union2);
        }
        return new MBeanOperationInfo(s, s, signature, canUseOpenInfo ? openReturnType.getClassName() : originalTypeString(genericReturnType), 3, union2);
    }
    
    @Override
    Descriptor getBasicMBeanDescriptor() {
        return new ImmutableDescriptor(new String[] { "mxbean=true", "immutableInfo=true" });
    }
    
    @Override
    Descriptor getMBeanDescriptor(Class<?> clazz) {
        return ImmutableDescriptor.EMPTY_DESCRIPTOR;
    }
    
    private static Descriptor typeDescriptor(OpenType<?> openType, Type type) {
        return new ImmutableDescriptor(new String[] { "openType", "originalType" }, new Object[] { openType, originalTypeString(type) });
    }
    
    private static boolean canUseOpenInfo(Type type) {
        if ((type instanceof GenericArrayType)) {
            return canUseOpenInfo(((GenericArrayType)type).getGenericComponentType());
        }
        if (type instanceof Class && ((Class)type).isArray()) {
            return canUseOpenInfo(((Class)type).getComponentType());
        }
        return !(type instanceof Class) || !((Class)type).isPrimitive();
    }
    
    private static String originalTypeString(Type type) {
        if ((type instanceof Class)) {
            return ((Class)type).getName();
        }
        return typeName(type);
    }
    
    static String typeName(Type type) {
        if ((type instanceof Class)) {
            Class<Object> clazz = (Class)type;
            if (clazz.isArray()) {
                return typeName(clazz.getComponentType()) + "[]";
            }
            return clazz.getName();
        }
        else {
            if ((type instanceof GenericArrayType)) {
                return typeName(((GenericArrayType)type).getGenericComponentType()) + "[]";
            }
            if ((type instanceof ParameterizedType)) {
                ParameterizedType parameterizedType = (ParameterizedType)type;
                StringBuilder sb = new StringBuilder();
                sb.append(typeName(parameterizedType.getRawType())).append("<");
                String str = "";
                Type[] actualTypeArguments = parameterizedType.getActualTypeArguments();
                for (int length = actualTypeArguments.length, i = 0; i < length; str = ", ", ++i) {
                    sb.append(str).append(typeName(actualTypeArguments[i]));
                }
                return sb.append(">").toString();
            }
            return "???";
        }
    }
    
    static {
        instance = new MXBeanIntrospector();
        mbeanInfoMap = new MBeanIntrospector.MBeanInfoMap();
    }
}
